<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV101-56-g411a2eed:2023-07-24#

namespace Platiti\PtiUniNs\UniModul;

use Exception;
use mysqli;
use mysqli_result;

interface IDatabaseConnection {
	/**
	 * @param string $sql
	 * @return array<int, array<string, mixed>>
	 */
	function sqlQuery($sql);
	/**
	 * @param string $sql
	 * @return void
	 */
	function sqlExecute($sql);
	/** @return int */
	function getInsertId();
}


class MysqliConnection implements IDatabaseConnection {

	/** @var mysqli */
	protected $mysqli;

	/**
	 * @param string|null $server
	 * @param string $login
	 * @param string $password
	 * @param string $dbname
	 */
	public function __construct($server, $login, $password, $dbname) {
		if ($server === null) {
			return; // situace pro zjisteni jen config info ve factory
		} else if (preg_match('/^(.*):([0-9]+)$/', $server, $matches)) {
			$host = $matches[1]; $port = (int)$matches[2];
			$this->mysqli = new mysqli($host, $login, $password, $dbname, $port);
		} elseif (preg_match('#^.*:(/.*)$#', $server, $matches)) {
			$socket = $matches[1];
			$this->mysqli = new mysqli('', $login, $password, $dbname, intval(ini_get("mysqli.default_port")), $socket);
		} elseif (preg_match('#^(/.*)$#', $server, $matches)) {
			$socket = $matches[1];
			$this->mysqli = new mysqli('', $login, $password, $dbname, intval(ini_get("mysqli.default_port")), $socket);
		} else {
			$this->mysqli = new mysqli($server, $login, $password, $dbname);
		}
		$this->mysqli->set_charset("utf8");
		$GLOBALS['DatabaseConnection_toSql_object'] = $this;
	}


	public function sqlQuery($sql) {
		$res = $this->mysqli->query($sql);
		if ($res === false) {
			throw new Exception("Query Failed! SQL: $sql - Error: ".$this->mysqli->error);
		}
		$ar = array();
		if ($res !== true) {
			while($row = $res->fetch_array()) {
				$ar[] = $row;
			}
			$res->free();
		}
		return $ar;
	}


	public function sqlExecute($sql) {
		$res = $this->mysqli->query($sql);
		if (!$res) throw new Exception("Query Failed! SQL: $sql - Error: ".$this->mysqli->error);
		if ($res instanceof mysqli_result) $res->free();
	}

	function getInsertId() {
		$id = $this->mysqli->insert_id;
		if ($id != (int)$id) throw new Exception('Non int insert_id');
		return (int)$id;
	}

	/**
	 * @param scalar|null $text
	 * @return string
	 */
	function toSql($text) {
		if (is_null($text)) {
			return "null";
		} else {
			return "'".$this->mysqli->real_escape_string(strval($text))."'";
		}
	}

}


class MysqliConnectionShared extends MysqliConnection {

	/**
	 * @param string|null $server
	 * @param string $login
	 * @param string $password
	 * @param string $dbname
	 */
	public function __construct($server, $login, $password, $dbname) {
		global $MysqliConnectionShared_mysqli;
		if ($MysqliConnectionShared_mysqli != null) {
			$this->mysqli = $MysqliConnectionShared_mysqli;
		} else {
			parent::__construct($server, $login, $password, $dbname);
			$MysqliConnectionShared_mysqli = $this->mysqli;
		}
	}
}


/**
 * @param scalar|null $text
 * @return string
 */
function toSql($text) {
	return $GLOBALS['DatabaseConnection_toSql_object']->toSql($text);
}

