<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV101-56-g411a2eed:2023-07-24#

require_once("UniModul.php");
require_once("UniAdapterModel.php");

use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;

if (interface_exists('Magento\Framework\App\CsrfAwareActionInterface')) {
	require_once(dirname(__FILE__).'/ActionCsrfAllowed.php');
} else {
	class ActionCsrfAllowed extends \Magento\Framework\App\Action\Action {
		public function execute()
		{
		}
	}
}

class UniAdapterController extends ActionCsrfAllowed {
	protected $uniModulName = null; //predefinuje binder
	protected $uniModul = null;
	protected $subMethod = '';


	protected $_context;
	protected $_checkoutSession;
	protected $_order;
	protected $_statusCollectionFactory;
	protected $_scopeConfig;
	protected $_storeManager;
	protected $_storeInterface;
	protected $_registry;
	protected $_extensionAttributesFactory;
	protected $_attributeValueFactory;
	protected $_helperData;
	protected $_magentoLogger;
	protected $_abstractResource;
	protected $_abstractDb;
	protected $_objectManager;
	protected $_resultPageFactory;


	public function __construct(
		\Magento\Framework\App\Action\Context $context,
		\Magento\Sales\Model\Order $_order,
		\Magento\Checkout\Model\Session $checkoutSession,
		\Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory $statusCollectionFactory,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Store\Api\Data\StoreInterface $storeInterface,
		\Magento\Framework\View\Result\PageFactory $resultPageFactory
	) {
		$this->_checkoutSession = $checkoutSession;
		$this->statusCollectionFactory = $statusCollectionFactory;
		$this->_order = $_order;
		$this->_scopeConfig = $scopeConfig;
		$this->_storeManager = $storeManager;
		$this->_storeInterface = $storeInterface;
		$this->_objectManager = Magento\Framework\App\ObjectManager::getInstance();
		$this->_resultPageFactory = $resultPageFactory;


		return parent::__construct($context);
	}

	function startsWith($haystack, $needle) {
		$length = strlen($needle);
		return (substr($haystack, 0, $length) === $needle);
	}

	public function execute() {
		try {
			BeginUniErr();
			$action = strtolower(explode("binder/Index/Index" . strtolower($this->subMethod), $_SERVER['REQUEST_URI'])[1]);
			$action = str_replace("/", "", $action);

			if ($this->startsWith($action, "redirectaction")) {
				return EndUniErr($this->redirectAction());
			} elseif ($this->startsWith($action, "reply")) {
				$this->replyAction();
			} elseif ($this->startsWith($action, "notify")) {
				$this->notifyAction();
			} elseif ($this->startsWith($action, "callback")) {
				$this->callbackAction();
			}
			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}

	}

	protected function initUniModul() {
		if ($this->uniModul == null) {
			if (is_null($this->uniModulName)) return;
			$uniFact = new UniModulFactory();
			$configSetting = $this->getConfigDataUni($this->uniModulName, $this->subMethod);
			$configSetting->uniModulConfig->funcProcessReplyStatus = array($this, 'funcProcessReplyStatus');
			$this->uniModul = $uniFact->createUniModul($this->uniModulName, $configSetting, $this->subMethod);
		}
	}

	protected function getConfigDataUni($uniModulName, $subMethod) {
		//payment abstractmethod kterou uniadaptermodel extenduje vyžaduje nějaký třídy, bez nich se nezobrazí
		//plateba na pokladně, při volání z uniadapteru to ale nepotřebujeme
		$model = new UniAdapterModel(null, null, null, null, null, null, null, null, null, array(),
			$uniModulName, $subMethod);
		return $model->getConfigDataUniExternal($uniModulName, $subMethod);
	}

	public function getForexMessage() {
		$store = $this->_objectManager->get("Magento\Framework\Locale\ResolverInterface");
		$model = new UniAdapterModel(null, null, null, null, null, null, null, null, null, array(),
			$this->uniModulName, $this->subMethod);
		$quote = $this->_checkoutSession->getQuote();
		list($isPossile, $forexNote) = $model->isAvailable($quote);
		return $forexNote;

	}

	public function redirectAction() {
    	BeginUniErr();
		$subMethod = $this->subMethod;

		$orderId = $this->_checkoutSession->getLastRealOrderId();
		$this->initUniModul();
		if ($orderId == null) { // https://www.tigren.com/magento-2-error-getlastrealorderid-doesnt-work/
			$this->uniModul->logger->writeLog("ADAPTER WARN: getLastRealOrderId returned null, clearing caches and retrying");
			$types = array('config','layout','block_html','collections','reflection','db_ddl','eav','config_integration','config_integration_api','full_page','translate','config_webservice');
			$cacheTypeList = $this->_objectManager->get("\Magento\Framework\App\Cache\TypeListInterface");
			foreach ($types as $type) {
				$cacheTypeList->cleanType($type);
			}
			$cacheFrontendPool = $this->_objectManager->get("\Magento\Framework\App\Cache\Frontend\Pool");
			foreach ($cacheFrontendPool as $cacheFrontend) {
				$cacheFrontend->getBackend()->clean();
			}
			$orderId = $this->_checkoutSession->getLastRealOrderId();
		}
		$order = $this->_order->loadByIncrementId($orderId);
		$orderToPayInfo = $this->getOrderToPayInfo($order, $subMethod);

		$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);

		// presmerovat na branu
		$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);

		if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
			$frontend_redir = $redirectAction->redirectUrl == null && $redirectAction->redirectForm == null;
			$this->processReplyStatus($redirectAction->orderReplyStatus, $frontend_redir);
		}

		if ($redirectAction->redirectUrl != null) {
			$resultFactory = $this->_objectManager->get('Magento\Framework\Controller\ResultFactory');
			$resultRedirect = $resultFactory->create('redirect');
			$resultRedirect->setUrl($redirectAction->redirectUrl);
			return EndUniErr($resultRedirect);
		} else if ($redirectAction->inlineForm != null) {
			$resultPage = $this->_resultPageFactory->create();
			$resultPage->getLayout()->initMessages();
			$resultPage->getLayout()->getBlock("Index.Index." . strtolower($this->uniModulName . "-" . $this->subMethod))->setInlineContent($redirectAction->inlineForm);
			return EndUniErr($resultPage);
		} else if ($redirectAction->redirectForm != null) {
			$this->uniModul->formRedirect($redirectAction->redirectForm);
		}
    	EndUniErr();
	}

	public function replyAction() {
    	BeginUniErr();
		$this->initUniModul();
		$orderReplyStatus = $this->uniModul->gatewayReceiveReply($this->getLanguage());
		$this->processReplyStatus($orderReplyStatus, true);
		EndUniErr();
	}

	public function notifyAction() {
    	BeginUniErr();
		$this->initUniModul();
		$orderReplyStatus = $this->uniModul->gatewayReceiveNotification();
		$this->processReplyStatus($orderReplyStatus, false);
    	EndUniErr();
	}

	public function callbackAction() {
    	BeginUniErr();
		$this->initUniModul();
		parse_str($_SERVER['QUERY_STRING'], $cbArgs);
		$this->uniModul->processCallbackRequest($_GET['_callbackName'], $cbArgs);
    	EndUniErr();
	}

	protected function getOrderToPayInfo($order, $subMethod) {

		$orderToPayInfo = new OrderToPayInfo();

		$orderToPayInfo->subMethod = $subMethod;

		$order_id = $order->getRealOrderId();
		$billing = $order->getBillingAddress();
		if ($billing == null) {
			$billing = $order->getShippingAddress();
		}

		$orderToPayInfo->shopPairingInfo = $order_id;
		$orderToPayInfo->shopOrderNumber = $order_id;

		$orderToPayInfo->amount = $order->getGrandTotal();
		$orderToPayInfo->currency = $order->getOrderCurrencyCode();
		$orderToPayInfo->uniAdapterData = array('order_total'=>$order->getBaseGrandTotal());

		$description = $this->_storeManager->getStore()->getFrontendName();
		$products = array();
		$cartItems = [];
		foreach ($order->getAllVisibleItems() as $item) {
			$products[] = $item->getName();

			$cartItem = new UniCartItem();
			$cartItem->type = UniCartItemType::commodity;
			$cartItem->name = $item->getName();
			$cartItem->quantity = $item->getQtyToShip();
			$cartItem->unitPrice = $item->getPriceInclTax();
			$cartItem->taxRate = round($item->getTaxPercent());
			$cartItems[] = $cartItem;
		}
		$orderToPayInfo->description = $description;

		$cartItem = new UniCartItem();
		$cartItem->type = UniCartItemType::delivery;
		$cartItem->name = 'shipping / doprava';
		$cartItem->quantity = 1;
		$cartItem->unitPrice = $order->getShippingInclTax();
		$cartItem->taxRate = $order->getShippingAmount() == 0 ? 0 : round(100 * $order->getShippingTaxAmount() / $order->getShippingAmount());
		$cartItems[] = $cartItem;

		$discountAmount = $order->getDiscountAmount();
		if ($discountAmount != 0) {
			// secteni pres sazby
			$rozdeleni = array_reduce($cartItems, function ($a, $ci) {
				$a[(string)$ci->taxRate] = $ci->unitPrice * $ci->quantity + (isset($a[$ci->taxRate]) ? $a[$ci->taxRate] : 0);
				return $a;
			}, array());


			// rovnomerne rozpocteni slevy na pasma
			$totalBeforeDiscount = $orderToPayInfo->amount - $discountAmount;
			foreach ($rozdeleni as $tax => $amt) {
				$cartItem = new UniCartItem();
				$cartItem->type = UniCartItemType::discount;
				$cartItem->name = 'discount / sleva';
				$cartItem->quantity = 1;
				$cartItem->unitPrice = $discountAmount * $amt / $totalBeforeDiscount;
				$cartItem->taxRate = $tax;
				$cartItems[] = $cartItem;
			}


		}


		$orderToPayInfo->cartItems = $cartItems;

		$customerData = new CustomerData();
		try {
			$customerData->first_name = ($billing->getFirstname() ? $billing->getFirstname() : $order->getCustomerFirstname());
			$customerData->last_name = ($billing->getLastname() ? $billing->getLastname() : $order->getCustomerLastname());
			$customerData->email = ($billing->getEmail() ? $billing->getEmail() : $order->getCustomerEmail());
		} catch (Exception $ex) {
			$customerData->first_name = ($order->getCustomerFirstname());
			$customerData->last_name = ($order->getCustomerLastname());
			$customerData->email = ($order->getCustomerEmail());

		}
		$customerData->street = implode(", ", $billing->getStreet());
		$customerData->city = $billing->getCity();
		$customerData->post_code = $billing->getPostcode();
		$customerData->country = $billing->getData()['country_id'];
		$customerData->phone = $billing->getTelephone();
		$customerData->identifier = $billing->getCustomer_id();  // pro guesta je to null
		$orderToPayInfo->customerData = $customerData;


		$code = ucfirst(strtolower($this->uniModulName)) . 'binder';
		$baseUrl = $this->_storeManager->getStore()->getBaseUrl();
		$orderToPayInfo->notifyUrl = $baseUrl . $code . '/Index/Index/notify';
		$orderToPayInfo->replyUrl = $baseUrl . $code . '/Index/Index/reply';
		$orderToPayInfo->uniModulDirUrl = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB) . "UniModul/";


		$orderToPayInfo->language = $this->getLanguage();

		$model = new UniAdapterModel();//$this->_modelContext, $this->_registry, $this->_extensionAttributesFactory,
		//$this->_attributeValueFactory, $this->_helperData, $this->_scopeConfig, $this->_magentoLogger);
		$orderToPayInfo->currencyRates = $model->getCurrencyRates();

		//$orderToPayInfo->uniAdapterData = $session->getId ();  //mozna bych tam mohl strcit celou session i s daty

		return $orderToPayInfo;
	}

	public function funcProcessReplyStatus($orderReplyStatus) {
				$this->processReplyStatus($orderReplyStatus, false);
	}

	function processReplyStatus($orderReplyStatus, $frontend_redir) {

		$cfg = $this->getConfigDataUni($this->uniModul->name, $this->uniModul->subMethod)->configData;

		$order = $this->_objectManager->get('Magento\Sales\Model\Order');

		switch ($orderReplyStatus->orderStatus)
		{
			case OrderStatus::$successful:
				$newState = $order::STATE_PROCESSING;
				$newStatus = $cfg['orderStatusSuccessfull'];
				$guiOk = true;
				break;
			case OrderStatus::$pending:
				$newState = $order::STATE_PENDING_PAYMENT;
				$newStatus = $cfg['orderStatusPending'];
				$guiOk = true;
				break;
			case OrderStatus::$failedFinal:
			case OrderStatus::$failedRetriable;
				$newState = $order::STATE_CANCELED;
				$newStatus = $cfg['orderStatusFailed'];
				$guiOk = false;
				break;
			case OrderStatus::$invalidReply;
				$newStatus = null;
				$newState = null;
				break;
			default:
				$this->uniModul->logger->writeLog("ADAPTER: unknown orderStatus: " . $orderReplyStatus->orderStatus);
				$newStatus = null;
				$newState = null;
				user_error("ADAPTER: unknown orderStatus: " . $orderReplyStatus->orderStatus);
		}

		BeginSynchronized();

		$order->loadByIncrementId($orderReplyStatus->shopPairingInfo);

		if (!$order->getId()) {
			// TODO  nacteni se nepovedlo...
			EndSynchronized();
			user_error("nacteni se nepovedlo");
			return;
		}


		if ($newStatus == null) {
			$this->uniModul->logger->writeLog("ADAPTER: invalidReply => no effect");
		} else {
			if ($order->getState() == $newState || !in_array($order->getState(), array($order::STATE_NEW, $order::STATE_PENDING_PAYMENT, $order::STATE_CANCELED))) {
				$this->uniModul->logger->writeLog("ADAPTER: Not updating shop order number ".$order->getRealOrderId().", old order state ".$order->getState().", new order state ".$newState);
			} else {

				$comment = $this->uniModul->dictionary->get('orderDetailGwOrdNum', $this->getLanguage()).': '.$orderReplyStatus->gwOrderNumber;;
				if ($orderReplyStatus->forexNote !== null) {
					$comment = "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $this->getLanguage()). ': ' . $orderReplyStatus->forexNote;
				}

				$payment = $order->getPayment();
				$payment->setTransactionId($orderReplyStatus->gwOrderNumber);
				$payment->setIsTransactionClosed(0);

				if ($newState == $order::STATE_PROCESSING) {

					$payment->registerCaptureNotification($orderReplyStatus->uniAdapterData['order_total']);
					if ($comment != "") {
						$payment->setPreparedMessage($comment);
					}

					$invoice = $payment->getCreatedInvoice();
					if ($invoice && !$order->getEmailSent()) {
						$this->uniModul->logger->writeLog("ADAPTER: Sending email - OrderSender");
						$this->_objectManager->get('Magento\Sales\Model\Order\Email\Sender\OrderSender')->send($order);
					}

				} else if ($newState == $order::STATE_CANCELED) {
					$order->registerCancellation($comment);
				}

				$this->uniModul->logger->writeLog("ADAPTER: Updating shop order number ".$order->getRealOrderId().", old order state ".$order->getState().", new order state ".$newState. ", new order status: " . $newStatus);
				$order->setState($newState);
				$history = $order->addCommentToStatusHistory($comment, $newStatus, true);
                $history->setIsVisibleOnFront(true);
                $history->setIsCustomerNotified(true);
				$order->save();
				$this->uniModul->logger->writeLog("ADAPTER: Sending email - OrderCommentSender");
				$this->_objectManager->get('\Magento\Sales\Model\Order\Email\Sender\OrderCommentSender')->send($order, true, $comment);

			}
		}

		EndSynchronized();


		if ($frontend_redir) {
			if ($orderReplyStatus->orderStatus == OrderStatus::$pending || $orderReplyStatus->orderStatus == OrderStatus::$successful) {
				//Mage::getSingleton('core/session')->addNotice('Order OK.');
				$this->_checkoutSession->setData('UniModulSuccessHtml', $orderReplyStatus->successHtml);
				$this->_redirect('checkout/onepage/success', array('_secure' => true));
			} else {
				$this->_checkoutSession->restoreQuote();
				$failMsg = __('Payment failed. Please try again.') . " " . $orderReplyStatus->resultText;

				$this->_objectManager->get('\Magento\Framework\Message\ManagerInterface')->addError($failMsg);

				$this->_redirect('checkout/cart');
			}
		}

	}

	public function getSuccessHtml() {
		return $this->_checkoutSession->getData('UniModulSuccessHtml');
	}

	protected function getLanguage() {
		$store = $this->_objectManager->get("Magento\Framework\Locale\ResolverInterface");
		return substr($store->getLocale(), 0, 2);
	}


}
