<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV101-56-g411a2eed:2023-07-24#

require_once("UniModul.php");
use Magento\Framework\Config\ConfigOptionsListConstants;

class UniAdapterModel extends \Magento\Payment\Model\Method\AbstractMethod {
	protected $uniModulName = null; //predefinuje binder
	protected $uniModul = null;
	protected $subMethod = '';


	protected $_code = '';
	//protected $_shouldInactivateQuote = false;   //nefunguje pro zachovani kosiku
	protected $_isGateway = true;
	protected $_isInitializeNeeded = true;
	protected $_formBlockType = 'platiti/form';
	protected $_canCapture = true;
	protected $_objectManager;
	protected $_modelContext;


	public function __construct(
		\Magento\Framework\Model\Context $context = null,
		\Magento\Framework\Registry $registry = null,
		\Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory = null,
		\Magento\Framework\Api\AttributeValueFactory $customAttributeFactory = null,
		\Magento\Payment\Helper\Data $paymentData = null,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig = null,
		\Magento\Payment\Model\Method\Logger $logger = null,
		\Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
		\Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
		array $data = [],
		$uniModulName = null,
		$subMethod = null
	) {
		//pokud je volán při zobrazování pokladny
		if($context != null || $registry != null || $data != null) {

			parent::__construct(
				$context,
				$registry,
				$extensionFactory,
				$customAttributeFactory,
				$resource,
				$resourceCollection,
				$data
			);
			$this->_paymentData = $paymentData;
			$this->_scopeConfig = $scopeConfig;
			$this->logger = $logger;
			$this->initializeData($data);

		} else { //pokud ho volám z uniadapteru
			$this->uniModulName = $uniModulName;
			$this->subMethod = $subMethod;
			$this->_objectManager = \Magento\Framework\App\ObjectManager::getInstance();

		}
	}

	// pro získani konfigurace pro controller, pro callback je totiz nutne aby byl adapter cely inicializovany
	public function getConfigDataUniExternal($uniModulName = null, $subMethod = null) {
		if ($uniModulName != null) $this->uniModulName = $uniModulName;
		if ($subMethod != null) $this->subMethod = $subMethod;
		return $this->initUniModul();
	}



	// jinak se nevola initialize
	public function getConfigPaymentAction() {
		BeginUniErr();
		$paymentAction = $this->getConfigData('payment_action');
		return EndUniErr(empty($paymentAction) ? true : $paymentAction);
	}

	protected function initUniModul() {
		if ($this->uniModul == null) {
			if (is_null($this->uniModulName)) return;
			$uniFact = new UniModulFactory();
			$configInfo = $uniFact->getConfigInfo($this->uniModulName, $this->getLanguage(), $this->subMethod);
			$configSetting = $this->getConfigDataUni($configInfo, $this->uniModulName, $this->subMethod);
			$this->uniModul = $uniFact->createUniModul($this->uniModulName, $configSetting, $this->subMethod);
			return $configSetting;
		}
	}




	public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null) {
		try {
			BeginUniErr();
			//u jednoho eshopu je stává že magento předá null místo quote
			if($quote == null){
				$checkoutSession = $this->_objectManager->get('\Magento\Checkout\Model\Session');
				$quote = $checkoutSession->getQuote();
			}


			$orderToPayInfo = $this->getOrderToPayInfo($quote, $this->subMethod);

			$this->initUniModul();

			//fix pro mmoduly se submetodama
			if ($this->subMethod != "") { //pokud se jedna o sub methodu
				//a pokud subMethodsSelection je prazdna (= nevyplnena v administraci, co z v magento neni)
				if (empty($this->uniModul->config->subMethodsSelection) || (count($this->uniModul->config->subMethodsSelection) == 1 && !$this->uniModul->config->subMethodsSelection[0])) {
					//tak to oblbneme
					$this->uniModul->config->subMethodsSelection = array($this->subMethod);
				}
			}

			$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);

			return EndUniErr(array($prePayGWInfo->isPossible, $prePayGWInfo->forexMessage));
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	protected function getConfigDataUni($configInfo, $uniModulName, $subMethod) {

		// doplnime failed stav, i kdyz ho modul nevyzaduje, protoze Magento vzdy vytvori order
		if (array_filter($configInfo->configFields, array("UniAdapterModel", "getConfigDataUni_filter")) === array()) {
			$configField = new ConfigField();
			$configField->name = 'orderStatusFailed';
			$configField->type = ConfigFieldType::$orderStatus;
			$configInfo->configFields[] = $configField;
		}

		$configData = array();
		$path = "payment/" . strtolower($uniModulName) . "binder/";

		$scopeConfig = $this->_objectManager->get('\Magento\Framework\App\Config\ScopeConfigInterface');
		foreach ($configInfo->configFields as $configField) {
			$configData[$configField->name] = $scopeConfig->getValue($path . $configField->name, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
		}



		$uniModulConfig = new UniModulConfig();


		$connConfig = $this->_objectManager->get('Magento\Framework\App\DeploymentConfig');

		$uniModulConfig->mysql_server = $connConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_CONNECTION_DEFAULT . '/host' );
		$uniModulConfig->mysql_dbname = $connConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_CONNECTION_DEFAULT . '/dbname' );
		$uniModulConfig->mysql_login = $connConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_CONNECTION_DEFAULT . '/username');
		$uniModulConfig->mysql_password = $connConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_CONNECTION_DEFAULT . '/password');

		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
		$uniModulConfig->uniModulDirUrl = $storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB) . 'UniModul/';
		$uniModulConfig->funcGetCallbackUrl = array($this, 'getCallbackUrl');

		$uniModulConfig->adapterName = 'Magento2';
		$cfgs = create_initialize_object('ConfigSetting', array('configData' => $configData, 'uniModulConfig' => $uniModulConfig));

		return $cfgs;

	}

	private static function getConfigDataUni_filter($e) {
		return $e->name == 'orderStatusFailed';
	}


	protected function getOrderToPayInfo($quote, $subMethod) {
		$orderToPayInfo = new OrderToPayInfo();

		$orderToPayInfo->subMethod = $subMethod;

		//$order_id = $quote->getRealOrderId();
		$billing = $quote->getBillingAddress();

		//$orderToPayInfo->shopPairingInfo = $order_id;
		//$orderToPayInfo->shopOrderNumber = $order_id;

		$orderToPayInfo->amount = $quote->getGrandTotal();
		$orderToPayInfo->currency = $quote->getQuoteCurrencyCode();

		$customerData = new CustomerData();
		$customerData->first_name = ($billing->getFirstname() ? $billing->getFirstname() : $quote->getCustomerFirstname());
		$customerData->last_name = ($billing->getLastname() ? $billing->getLastname() : $quote->getCustomerLastname());
		$customerData->email = ($billing->getEmail() ? $billing->getEmail() : $quote->getCustomerEmail());
		$customerData->street = $billing->getStreet(-1);
		$customerData->city = $billing->getCity();
		$customerData->post_code = $billing->getPostcode();
		$customerData->country = $billing->getCountryModel()->getIso3Code();
		$customerData->phone = $billing->getTelephone();
		$orderToPayInfo->customerData = $customerData;



		$orderToPayInfo->language = $this->getLanguage();

		$orderToPayInfo->currencyRates = $this->getCurrencyRates();


		return $orderToPayInfo;
	}

	public function getCurrencyRates() {
		$currencyRates = array();
		$storeManager = $this->_objectManager->get('\Magento\Store\Model\StoreManagerInterface');
		foreach ($storeManager->getStore()->getAvailableCurrencyCodes(false) as $availableCurrencyCode) {
			$currency = $this->_objectManager->create('Magento\Directory\Model\CurrencyFactory')->create()->load($availableCurrencyCode);
			$currencyRates[$availableCurrencyCode] = $storeManager->getStore()->getBaseCurrency()->getRate($availableCurrencyCode);
		}
		return $currencyRates;
	}

	public function getCallbackUrl($callbackName, $arguments) {
		$storeManager = $this->_objectManager->get('\Magento\Store\Model\StoreManagerInterface');
		$code = ucfirst(strtolower( $this->uniModulName)) . 'binder';
		$baseUrl = $storeManager->getStore()->getBaseUrl();
		return $baseUrl.$code.'/Index/Index/callback' . "?_unimodul=" . $this->uniModul->name . "&_callbackName=" . urlencode($callbackName) . "&" . http_build_query($arguments);;
	}

	function getMiniLogo() {
		return $this->uniModul->getMiniLogoSpan();
	}

	function getImage() {
		return $this->uniModul->getModulSubMethodLogoImage2($this->uniModul->subMethod);
	}


	public function getInstallmentHtml() {
		try {
			BeginUniErr();
			$this->_objectManager = \Magento\Framework\App\ObjectManager::getInstance();
			$productCollection = $this->_objectManager->create('Magento\Catalog\Model\ResourceModel\Product\CollectionFactory');
			$collection = $productCollection->create()
				->addAttributeToSelect('*')
				->addAttributeToSort('created_at', 'DESC')
				->setPageSize(8)
				->load();

			foreach ($collection as $currentProduct) {
				$this->initUniModul();

				$scopeConfig = $this->_objectManager->get('\Magento\Framework\App\Config\ScopeConfigInterface');
				$path = "payment/" . strtolower($this->uniModulName) . "binder/";
				if(!$scopeConfig->getValue($path . "active")){
					return(EndUniErr(''));
				}

				$storeManager = $this->_objectManager->create('\Magento\Store\Model\StoreManagerInterface');
				$price = $currentProduct->getFinalPrice();
				if ($storeManager->getStore()->getCurrentCurrency() && $storeManager->getStore()->getBaseCurrency()) {
					$product_price = $storeManager->getStore()->getBaseCurrency()->convert($price, $storeManager->getStore()->getCurrentCurrency());
				} else {
					$product_price = $price;
				}






				$current_currency = $storeManager->getStore()->getCurrentCurrency()->getCurrencyCode();

				$baseUrl = $storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);
				$embedHtml = $this->uniModul->ProductGetInstallmentEmbedHtml($baseUrl, $current_currency, $product_price);

				return EndUniErr($embedHtml);

			}
			return EndUniErr('');
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	protected function getLanguage() {
		$store = $this->_objectManager->get("Magento\Framework\Locale\ResolverInterface");
		return substr($store->getLocale(), 0, 2);
	}

}
